@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import functools as ft
 from collections import OrderedDict, defaultdict
-from typing import Any, Callable, Iterable, NamedTuple, Sequence
+from typing import Any, Callable, NamedTuple, Sequence
 
 import jax.tree_util as jtu
 from jax._src.tree_util import _registry
@@ -12,16 +12,31 @@ from jax._src.tree_util import _registry
 # the code style is heavilty influenced by `jax.tree_util`
 # https://github.com/google/jax/blob/main/jax/_src/tree_util.py
 
+
 PyTree = Any
 TraceType = Any
 
-_trace_registry: dict[type, _TraceRegistryEntry] = dict()
+
+def _jaxable_trace_func(tree: Any) -> list[TraceType]:
+    # fallback trace function in case no trace function is registered for a given
+    # class in the `trace` registry
+    # get leaves from the `jax` registry
+    leaves, _ = _registry.get(type(tree)).to_iter(tree)  # type: ignore
+    # TODO: fetch from `jax` key registry once min jax version>=0.4.6
+    names = (f"leaf_{i}" for i in range(len(leaves)))
+    types = map(type, leaves)
+    indices = range(len(leaves))
+    metadatas = (dict(id=id(leaf)) for leaf in leaves)
+    return [*zip(names, types, indices, metadatas)]
 
 
 class _TraceRegistryEntry(NamedTuple):
     to_iter: Callable[..., Any]
 
 
+_trace_registry = defaultdict(lambda: _TraceRegistryEntry(_jaxable_trace_func))
+
+
 def _validate_trace_func(
     trace_func: Callable[[Any], TraceType]
 ) -> Callable[[Any], TraceType]:
@@ -29,46 +44,46 @@ def _validate_trace_func(
     # validation is only performed once
     @ft.wraps(trace_func)
     def wrapper(tree):
-        if wrapper.is_validated:
+        if wrapper.has_run is True:
             return trace_func(tree)
 
         for trace in (traces := trace_func(tree)):
             # check if the trace has the correct format
             if not isinstance(trace, (list, tuple)):
-                msg = f"Trace return type is not defined properly for "
+                msg = "Trace return type is not defined properly for "
                 msg += f"class=`{type(tree).__name__}`."
                 msg += f"Expected a list or tuple, got {trace}"
                 raise TypeError(msg)
 
             if len(trace) != 4:
-                msg = f"Trace length is not defined properly for "
+                msg = "Trace length is not defined properly for "
                 msg += f"class=`{type(tree).__name__}`."
                 msg += "Expected 4 entries, in the order of"
                 msg += f"(name, type, index, metadata), got {trace}"
                 raise ValueError(msg)
 
             if not isinstance(trace[0], str):
-                msg = f"Trace name entry is not defined properly for "
+                msg = "Trace name entry is not defined properly for "
                 msg += f"class=`{type(tree).__name__}`."
                 msg += f" Expected a string, got {trace[0]}"
                 raise TypeError(msg)
 
             if not isinstance(trace[1], type):
-                msg = f"Trace type entry is not defined properly for "
+                msg = "Trace type entry is not defined properly for "
                 msg += f"class=`{type(tree).__name__}`."
                 msg += f" Expected a type, got {trace[1]}"
                 raise TypeError(msg)
 
             if not isinstance(trace[2], int):
-                msg = f"Trace index entry is not defined properly for "
+                msg = "Trace index entry is not defined properly for "
                 msg += f"class=`{type(tree).__name__}`."
                 msg += f" Expected an integer, got {trace[2]}"
                 raise TypeError(msg)
 
-        wrapper.is_validated = True
+        wrapper.has_run = True
         return traces
 
-    wrapper.is_validated = False
+    wrapper.has_run = False
     return wrapper
 
 
@@ -121,118 +136,6 @@ def register_pytree_node_trace(
     _trace_registry[klass] = _TraceRegistryEntry(_validate_trace_func(trace_func))
 
 
-def flatten_one_trace_level(
-    tree_trace: TraceType,
-    tree: PyTree,
-    is_leaf: Callable[[Any], bool] | None,
-    is_trace_leaf: Callable[[TraceType], bool] | None,
-):
-    if (is_leaf and is_leaf(tree)) or (is_trace_leaf and is_trace_leaf(tree_trace)):
-        # is_leaf is a predicate function that determines whether a value is a leaf
-        # is_trace_leaf is a predicate function that determines whether a trace is a leaf
-        yield tree_trace, tree
-        return
-
-    if type(tree) in _registry:
-        # trace handler for the current tree
-        leaves, _ = _registry[type(tree)].to_iter(tree)
-
-        # trace handler for the current tree
-        # defaults to `_jaxable_trace_func`
-        traces = (
-            _trace_registry[type(tree)].to_iter(tree)
-            if type(tree) in _trace_registry
-            else _jaxable_trace_func(tree)
-        )
-
-    elif isinstance(tree, tuple) and hasattr(tree, "_fields"):
-        # this conforms to the `jax` convention for namedtuples
-        leaves = (getattr(tree, field) for field in tree._fields)  # type: ignore
-        traces = _namedtuple_trace_func(tree)
-
-    elif tree is not None:
-        yield tree_trace, tree
-        return
-
-    for rhs_trace, leaf in zip(traces, leaves):
-        leaf_trace = (
-            (*tree_trace[0], rhs_trace[0]),  # names
-            (*tree_trace[1], rhs_trace[1]),  # types
-            (*tree_trace[2], rhs_trace[2]),  # indices
-            (*tree_trace[3], rhs_trace[3]),  # metadatas
-        )
-        yield from flatten_one_trace_level(leaf_trace, leaf, is_leaf, is_trace_leaf)
-
-
-def tree_leaves_with_trace(
-    tree: PyTree,
-    *,
-    is_leaf: Callable[[Any], bool] | None = None,
-    is_trace_leaf: Callable[[Any], bool] | None = None,
-) -> Sequence[tuple[TraceType, Any]]:
-    r"""Similar to jax.tree_util.tree_leaves` but returns  object, leaf pairs
-
-    Args:
-        tree: The tree to be flattened.
-        is_leaf: A predicate function that determines whether a value is a leaf.
-        is_trace_leaf: A predicate function that determines whether a trace is a leaf.
-
-    Returns:
-        A list of (trace, leaf) pairs.
-
-    Example:
-        >>> import pytreeclass as pytc
-        >>> tree = [1, [2, [3]]]
-        >>> traces, _ = zip(*pytc.tree_leaves_with_trace(tree))
-        >>> # print(pytc.tree_repr(traces))
-        >>> # (
-        >>> #   (
-        >>> #     ('list', '[0]'),                                                      # -> name path of leaf = 1
-        >>> #     (<class 'list'>, <class 'int'>),                                      # -> type path of leaf = 1
-        >>> #     (0, 0),                                                               # -> index path of leaf = 1
-        >>> #     ({id:4951960512}, {id:4380344560})                                    # -> metadata path of leaf = 1
-        >>> #   ),
-        >>> #   (
-        >>> #     ('list', '[1]', '[0]'),                                               # -> name path of leaf = 2
-        >>> #     (<class 'list'>, <class 'list'>, <class 'int'>),                      # -> type path of leaf = 2
-        >>> #     (0, 1, 0),                                                            # -> index path of leaf = 2
-        >>> #     ({id:4951960512}, {id:4951876032}, {id:4380344592})                   # -> metadata path of leaf = 2
-        >>> #   ),
-        >>> #   (
-        >>> #     ('list', '[1]', '[1]', '[0]'),                                        # -> name path of leaf = 3
-        >>> #     (<class 'list'>, <class 'list'>, <class 'list'>, <class 'int'>),      # -> type path of leaf = 3
-        >>> #     (0, 1, 1, 0),                                                         # -> index path of leaf = 3
-        >>> #     ({id:4951960512}, {id:4951876032}, {id:4950290624}, {id:4380344624})  # -> metadata path of leaf = 3
-        >>> #   )
-        >>> # )
-
-    Note:
-        `metadata` path can hold any information about the object. PyTreeClass stores the object id
-        for the common data structures like `list`, `tuple`, `dict`, `set`, `namedtuple`, and `treeclass` wrapped
-        classes.
-    """
-    trace = ((type(tree).__name__,), (type(tree),), (0,), (dict(id=id(tree)),))  # type: ignore
-    return list(flatten_one_trace_level(trace, tree, is_leaf, is_trace_leaf))
-
-
-def tree_flatten_with_trace(
-    tree: PyTree, *, is_leaf: Callable[[Any], bool] | None = None
-) -> tuple[Sequence[tuple[TraceType, Any]], jtu.PyTreeDef]:
-    """Similar to jax.tree_util.tree_flatten` but returns the objects too as well
-
-    Args:
-        tree: The tree to be flattened.
-        is_leaf: A predicate function that determines whether a value is a leaf.
-
-    Returns:
-        A pair (leaves, treedef) where leaves is a list of (trace, leaf) pairs and
-        treedef is a PyTreeDef object that can be used to reconstruct the tree.
-    """
-    treedef = jtu.tree_structure(tree, is_leaf=is_leaf)
-    traces_leaves = tree_leaves_with_trace(tree, is_leaf=is_leaf)
-    return traces_leaves, treedef
-
-
 def _sequence_trace_func(tree: Sequence) -> list[TraceType]:
     names = (f"[{i}]" for i in range(len(tree)))
     types = map(type, tree)
@@ -257,25 +160,117 @@ def _namedtuple_trace_func(tree: Any) -> list[TraceType]:
     return [*zip(names, types, indices, metadatas)]
 
 
-def _jaxable_trace_func(tree: Any) -> list[TraceType]:
-    # fallback trace function in case no trace function is registered for a given
-    # class in the `trace` registry
-    # get leaves from the `jax` registry
-    leaves, _ = _registry.get(type(tree)).to_iter(tree)  # type: ignore
-    # TODO: fetch from `jax` key registry once min jax version>=0.4.6
-    names = (f"leaf_{i}" for i in range(len(leaves)))
-    types = map(type, leaves)
-    indices = range(len(leaves))
-    metadatas = (dict(id=id(leaf)) for leaf in leaves)
-    return [*zip(names, types, indices, metadatas)]
-
-
 # register trace functions for common types
-_trace_registry[tuple] = _TraceRegistryEntry(_sequence_trace_func)
-_trace_registry[list] = _TraceRegistryEntry(_sequence_trace_func)
-_trace_registry[dict] = _TraceRegistryEntry(_dict_trace_func)
-_trace_registry[OrderedDict] = _TraceRegistryEntry(_dict_trace_func)
-_trace_registry[defaultdict] = _TraceRegistryEntry(_dict_trace_func)
+register_pytree_node_trace(tuple, _sequence_trace_func)
+register_pytree_node_trace(list, _sequence_trace_func)
+register_pytree_node_trace(dict, _dict_trace_func)
+register_pytree_node_trace(OrderedDict, _dict_trace_func)
+register_pytree_node_trace(defaultdict, _dict_trace_func)
+
+
+def flatten_one_trace_level(
+    tree_trace: TraceType,
+    tree: PyTree,
+    is_leaf: Callable[[Any], bool] | None,
+    is_trace_leaf: Callable[[TraceType], bool] | None,
+):
+    if (is_leaf and is_leaf(tree)) or (is_trace_leaf and is_trace_leaf(tree_trace)):
+        # is_leaf is a predicate function that determines whether a value is a leaf
+        # is_trace_leaf is a predicate function that determines whether a trace is a leaf
+        yield tree_trace, tree
+        return
+
+    if type(tree) in _registry:
+        # trace handler for the current tree
+        leaves, _ = _registry[type(tree)].to_iter(tree)
+
+        # trace handler for the current tree
+        # defaults to `_jaxable_trace_func`
+        traces = _trace_registry[type(tree)].to_iter(tree)
+
+    elif isinstance(tree, tuple) and hasattr(tree, "_fields"):
+        # this conforms to the `jax` convention for namedtuples
+        leaves = (getattr(tree, field) for field in tree._fields)  # type: ignore
+        traces = _namedtuple_trace_func(tree)
+
+    else:
+        yield tree_trace, tree
+        return
+
+    for rhs_trace, leaf in zip(traces, leaves):
+        leaf_trace = (
+            (*tree_trace[0], rhs_trace[0]),  # names
+            (*tree_trace[1], rhs_trace[1]),  # types
+            (*tree_trace[2], rhs_trace[2]),  # indices
+            (*tree_trace[3], rhs_trace[3]),  # metadatas
+        )
+        yield from flatten_one_trace_level(leaf_trace, leaf, is_leaf, is_trace_leaf)
+
+
+def tree_leaves_with_trace(
+    tree: PyTree,
+    *,
+    is_leaf: Callable[[Any], bool] | None = None,
+    is_trace_leaf: Callable[[Any], bool] | None = None,
+) -> Sequence[tuple[TraceType, Any]]:
+    r"""Similar to jax.tree_util.tree_leaves` but returns  object, leaf pairs.
+
+    Args:
+        tree: The tree to be flattened.
+        is_leaf: A predicate function that determines whether a value is a leaf.
+        is_trace_leaf: A predicate function that determines whether a trace is a leaf.
+
+    Returns:
+        A list of (trace, leaf) pairs.
+
+    Example:
+        >>> import pytreeclass as pytc
+        >>> tree = [1, [2, [3]]]
+        >>> traces, _ = zip(*pytc.tree_leaves_with_trace(tree))
+        >>> # print(pytc.tree_repr(traces))
+        >>> # (
+        >>> # (('[0]'), (<class 'int'>), (0), ({id:4383785200})),
+        >>> # (
+        >>> #     ('[1]', '[0]'),
+        >>> #     (<class 'list'>, <class 'int'>),
+        >>> #     (1, 0),
+        >>> #     ({id:4558856448}, {id:4383785232})
+        >>> # ),
+        >>> # (
+        >>> #     ('[1]', '[1]', '[0]'),
+        >>> #     (<class 'list'>, <class 'list'>, <class 'int'>),
+        >>> #     (1, 1, 0),
+        >>> #     ({id:4558856448}, {id:4558857472}, {id:4383785264})
+        >>> # )
+        >>> # )
+
+    Note:
+        `metadata` path can hold any information about the object. PyTreeClass stores the object id
+        for the common data structures like `list`, `tuple`, `dict`, `set`, `namedtuple`, and `treeclass` wrapped
+        classes.
+    """
+    trace = ((), (), (), ())
+    return list(flatten_one_trace_level(trace, tree, is_leaf, is_trace_leaf))
+
+
+def tree_flatten_with_trace(
+    tree: PyTree,
+    *,
+    is_leaf: Callable[[Any], bool] | None = None,
+) -> tuple[Sequence[tuple[TraceType, Any]], jtu.PyTreeDef]:
+    """Similar to jax.tree_util.tree_flatten` but returns the objects too as well
+
+    Args:
+        tree: The tree to be flattened.
+        is_leaf: A predicate function that determines whether a value is a leaf.
+
+    Returns:
+        A pair (leaves, treedef) where leaves is a list of (trace, leaf) pairs and
+        treedef is a PyTreeDef object that can be used to reconstruct the tree.
+    """
+    treedef = jtu.tree_structure(tree, is_leaf=is_leaf)
+    traces_leaves = tree_leaves_with_trace(tree, is_leaf=is_leaf)
+    return traces_leaves, treedef
 
 
 def tree_map_with_trace(
@@ -299,7 +294,7 @@ def tree_map_with_trace(
         >>> import jax
         >>> import pytreeclass as pytc
         >>> tree = {"a": [1, 2], "b": 4, "c": [5, 6]}
-
+        >>> # the tree above is visualized as:
         >>> # value tree:
         >>> #        tree
         >>> #          |
@@ -315,22 +310,22 @@ def tree_map_with_trace(
         >>> #          |
         >>> #    ---------------
         >>> #    |      |      |
-        >>> #   'a'    'b'    'c'
+        >>> # `['a']` `['b']``['c']`
         >>> #    |             |
         >>> #  ------       -------
         >>> #  |    |       |     |
         >>> #`[0]``[1]`   `[0]` `[1]`
 
         >>> # type tree:
-        >>> #        dict
+        >>> #        <dict>
         >>> #          |
         >>> #    ---------------
         >>> #    |      |      |
-        >>> #   list   int    list
+        >>> #  <list> <int> <list>
         >>> #    |             |
         >>> #  ------       -------
         >>> #  |    |       |     |
-        >>> # int  int     int   int
+        >>> # <int><int>   <int><int>
 
         >>> # index tree:
         >>> #          0
